indexing
   description: "childrens having children and an optional parent"

class COMPOSITE [T]

--------------------------
feature -- Maintenance

parent : COMPOSITE [T] is do end
  -- Optional parent, default is Void. If an explicit reference
  -- to the parent is needed, redefine as an attribute.
		
has (child : T) : BOOLEAN is
  -- does 'child' belong to the composite?
  require
    child_not_void: child /= Void
  do
    Result := children.has(child)
  end

add (new_child : T) is
  -- add `new_child' to the composite
  require
    new_child_not_void: new_child /= Void
  do
    children.put_front(new_child)
  ensure
    added: has(new_child)
  end
		
remove (child : T) is
  -- remove child from the composite
  require
    child_not_void: child /= Void
  do
    from children.start
    until children.after or child = children.item
    loop
    	children.forth
    end

    if not children.after then children.remove end
  ensure
    removed: not has(child)
  end
		
--------------------------
feature {NONE} -- Attributes

children : LINKED_LIST [T] 

invariant
  children_not_void: children /= Void

end
